/* charset.h - Maintain a character set for a crossword puzzle
 *
 * Copyright 2021 Federico Mena Quintero <federico@gnome.org>
 * Copyright 2023 Jonathan Blandford <jrb@gnome.org>
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * SPDX-License-Identifier: (LGPL-2.1-or-later OR MIT)
 */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS


#define IPUZ_TYPE_CHARSET (ipuz_charset_get_type ())
#define IPUZ_CHARSET(charset) ((IPuzCharset *)charset)

typedef struct _IPuzCharset IPuzCharset;
typedef struct _IPuzCharsetIter IPuzCharsetIter;
typedef struct _IPuzCharsetIterValue
{
  gunichar c;
  guint count;
} IPuzCharsetIterValue;


IPuzCharset          *ipuz_charset_new              (void);
IPuzCharset          *ipuz_charset_new_from_text    (const char        *text);
IPuzCharset          *ipuz_charset_new_for_language (const char        *lang);
IPuzCharset          *ipuz_charset_ref              (IPuzCharset       *charet);
void                  ipuz_charset_unref            (IPuzCharset       *charset);
gboolean              ipuz_charset_compare          (IPuzCharset       *charset_a,
                                                     IPuzCharset       *charset_b);
void                  ipuz_charset_add_text         (IPuzCharset       *charset,
                                                     const char        *text);
void                  ipuz_charset_add_character    (IPuzCharset       *charset,
                                                     gunichar           c);
gint                  ipuz_charset_get_char_index   (const IPuzCharset *charset,
                                                     gunichar           c);
guint                 ipuz_charset_get_char_count   (const IPuzCharset *charset,
                                                     gunichar           c);
gsize                 ipuz_charset_get_n_chars      (const IPuzCharset *charset);
gsize                 ipuz_charset_get_size         (const IPuzCharset *charset);
IPuzCharsetIter      *ipuz_charset_iter_first       (const IPuzCharset *ipuz_charset);
IPuzCharsetIter      *ipuz_charset_iter_next        (IPuzCharsetIter   *iter);
IPuzCharsetIterValue  ipuz_charset_iter_get_value   (IPuzCharsetIter   *iter);
gchar                *ipuz_charset_serialize        (const IPuzCharset *ipuz_charset);
IPuzCharset          *ipuz_charset_deserialize      (const char        *str);
IPuzCharset          *ipuz_charset_clone            (const IPuzCharset *ipuz_charset);
gboolean              ipuz_charset_contains_text    (const IPuzCharset *ipuz_charset,
                                                     const char        *text);
gboolean              ipuz_charset_remove_text      (IPuzCharset       *ipuz_charset,
                                                     const char        *text);


G_DEFINE_AUTOPTR_CLEANUP_FUNC (IPuzCharset, ipuz_charset_unref);


G_END_DECLS
